package javax.swing.text;

import java.awt.geom.Rectangle2D;
import java.awt.Graphics2D;
import java.awt.Shape;
import java.awt.Graphics;
import java.awt.font.TextHitInfo;
import java.awt.font.TextLayout;

class GlyphPainter2 extends GlyphView.GlyphPainter
{
    TextLayout layout;
    
    public GlyphPainter2(TextLayout layout) {
        this.layout = layout;
    }
    
    @Override
    public GlyphView.GlyphPainter getPainter(GlyphView glyphView, int n, int n2) {
        return null;
    }
    
    @Override
    public float getSpan(GlyphView glyphView, int n, int n2, TabExpander tabExpander, float n3) {
        if (n == glyphView.getStartOffset() && n2 == glyphView.getEndOffset()) {
            return this.layout.getAdvance();
        }
        int startOffset = glyphView.getStartOffset();
        int offset = n - startOffset;
        int offset2 = n2 - startOffset;
        TextHitInfo afterOffset = TextHitInfo.afterOffset(offset);
        TextHitInfo beforeOffset = TextHitInfo.beforeOffset(offset2);
        float n4 = this.layout.getCaretInfo(afterOffset)[0];
        float n5 = this.layout.getCaretInfo(beforeOffset)[0];
        return (n5 > n4) ? (n5 - n4) : (n4 - n5);
    }
    
    @Override
    public float getHeight(GlyphView glyphView) {
        return this.layout.getAscent() + this.layout.getDescent() + this.layout.getLeading();
    }
    
    @Override
    public float getAscent(GlyphView glyphView) {
        return this.layout.getAscent();
    }
    
    @Override
    public float getDescent(GlyphView glyphView) {
        return this.layout.getDescent();
    }
    
    @Override
    public void paint(GlyphView glyphView, Graphics graphics, Shape shape, int n, int n2) {
        if ((graphics instanceof Graphics2D)) {
            Rectangle2D bounds2D = shape.getBounds2D();
            Graphics2D graphics2D = (Graphics2D)graphics;
            float n3 = (float)bounds2D.getY() + this.layout.getAscent() + this.layout.getLeading();
            float n4 = (float)bounds2D.getX();
            if (n <= glyphView.getStartOffset() && n2 >= glyphView.getEndOffset()) {
                this.layout.draw(graphics2D, n4, n3);
                return;
            }
            try {
                Shape modelToView = glyphView.modelToView(n, Position.Bias.Forward, n2, Position.Bias.Backward, shape);
                Shape clip = graphics.getClip();
                graphics2D.clip(modelToView);
                this.layout.draw(graphics2D, n4, n3);
                graphics.setClip(clip);
            }
            catch (BadLocationException ex) {}
        }
    }
    
    @Override
    public Shape modelToView(GlyphView glyphView, int n, Position.Bias bias, Shape shape) throws BadLocationException {
        int n2 = n - glyphView.getStartOffset();
        Rectangle2D bounds2D = shape.getBounds2D();
        bounds2D.setRect(bounds2D.getX() + (double)this.layout.getCaretInfo((bias == Position.Bias.Forward) ? TextHitInfo.afterOffset(n2) : TextHitInfo.beforeOffset(n2))[0], bounds2D.getY(), 1.0, bounds2D.getHeight());
        return bounds2D;
    }
    
    @Override
    public int viewToModel(GlyphView glyphView, float n, float n2, Shape shape, Position.Bias[] array) {
        TextHitInfo hitTestChar = this.layout.hitTestChar(n - (float)((shape instanceof Rectangle2D) ? ((Rectangle2D)shape) : shape.getBounds2D()).getX(), 0.0f);
        int insertionIndex = hitTestChar.getInsertionIndex();
        if (insertionIndex == glyphView.getEndOffset()) {
            --insertionIndex;
        }
        array[0] = (hitTestChar.isLeadingEdge() ? Position.Bias.Forward : Position.Bias.Backward);
        return insertionIndex + glyphView.getStartOffset();
    }
    
    @Override
    public int getBoundedPosition(GlyphView glyphView, int n, float n2, float x) {
        if (x < 0.0f) {
            throw new IllegalArgumentException("Length must be >= 0.");
        }
        TextHitInfo textHitInfo;
        if (this.layout.isLeftToRight()) {
            textHitInfo = this.layout.hitTestChar(x, 0.0f);
        }
        else {
            textHitInfo = this.layout.hitTestChar(this.layout.getAdvance() - x, 0.0f);
        }
        return glyphView.getStartOffset() + textHitInfo.getCharIndex();
    }
    
    @Override
    public int getNextVisualPositionFrom(GlyphView glyphView, int n, Position.Bias bias, Shape shape, int i, Position.Bias[] array) throws BadLocationException {
        Document document = glyphView.getDocument();
        int startOffset = glyphView.getStartOffset();
        int endOffset = glyphView.getEndOffset();
        switch (i) {
            case 1: {
                break;
            }
            case 5: {
                break;
            }
            case 3: {
                int leftToRight = AbstractDocument.isLeftToRight(document, startOffset, endOffset);
                if (startOffset == document.getLength()) {
                    if (n == -1) {
                        array[0] = Position.Bias.Forward;
                        return startOffset;
                    }
                    return -1;
                }
                else if (n == -1) {
                    if (!(leftToRight == 0)) {
                        array[0] = Position.Bias.Forward;
                        return startOffset;
                    }
                    Segment text2 = glyphView.getText(endOffset - 1, endOffset);
                    int n3 = text2.array[text2.offset];
                    SegmentCache.releaseSharedSegment(text2);
                    if (n3 == '\n') {
                        array[0] = Position.Bias.Forward;
                        return endOffset - 1;
                    }
                    array[0] = Position.Bias.Backward;
                    return endOffset;
                }
                else {
                    TextHitInfo hit;
                    if (bias == Position.Bias.Forward) {
                        hit = TextHitInfo.afterOffset(n - startOffset);
                    }
                    else {
                        hit = TextHitInfo.beforeOffset(n - startOffset);
                    }
                    TextHitInfo hit2 = this.layout.getNextRightHit(hit);
                    if (hit2 == null) {
                        return -1;
                    }
                    if (leftToRight != (this.layout.isLeftToRight() ? 1 : 0)) {
                        hit2 = this.layout.getVisualOtherHit(hit2);
                    }
                    n = hit2.getInsertionIndex() + startOffset;
                    if (n == endOffset) {
                        Segment text = glyphView.getText(endOffset - 1, endOffset);
                        int n2 = text.array[text.offset];
                        SegmentCache.releaseSharedSegment(text);
                        if (n2 == '\n') {
                            return -1;
                        }
                        array[0] = Position.Bias.Backward;
                    }
                    else {
                        array[0] = Position.Bias.Forward;
                    }
                    return n;
                }
            }
            case 7: {
                int leftToRight2 = AbstractDocument.isLeftToRight(document, startOffset, endOffset);
                if (startOffset == document.getLength()) {
                    if (n == -1) {
                        array[0] = Position.Bias.Forward;
                        return startOffset;
                    }
                    return -1;
                }
                else if (n == -1) {
                    if (!(leftToRight2 == 0)) {
                        Segment text4 = glyphView.getText(endOffset - 1, endOffset);
                        int ch = text4.array[text4.offset];
                        SegmentCache.releaseSharedSegment(text4);
                        if (ch == '\n' || Character.isSpaceChar((char)ch)) {
                            array[0] = Position.Bias.Forward;
                            return endOffset - 1;
                        }
                        array[0] = Position.Bias.Backward;
                        return endOffset;
                    }
                    else {
                        array[0] = Position.Bias.Forward;
                        return startOffset;
                    }
                }
                else {
                    TextHitInfo hit3;
                    if (bias == Position.Bias.Forward) {
                        hit3 = TextHitInfo.afterOffset(n - startOffset);
                    }
                    else {
                        hit3 = TextHitInfo.beforeOffset(n - startOffset);
                    }
                    TextHitInfo hit4 = this.layout.getNextLeftHit(hit3);
                    if (hit4 == null) {
                        return -1;
                    }
                    if (leftToRight2 != (this.layout.isLeftToRight() ? 1 : 0)) {
                        hit4 = this.layout.getVisualOtherHit(hit4);
                    }
                    n = hit4.getInsertionIndex() + startOffset;
                    if (n == endOffset) {
                        Segment text3 = glyphView.getText(endOffset - 1, endOffset);
                        int n4 = text3.array[text3.offset];
                        SegmentCache.releaseSharedSegment(text3);
                        if (n4 == '\n') {
                            return -1;
                        }
                        array[0] = Position.Bias.Backward;
                    }
                    else {
                        array[0] = Position.Bias.Forward;
                    }
                    return n;
                }
            }
            default: {
                throw new IllegalArgumentException("Bad direction: " + i);
            }
        }
        return n;
    }
}
